; 640x480x16 graphics
; Jan Wassenberg

global h_line
global v_line


section CODE use16

;
; h_line(x1, x2, y, color : word)
; destroys:
;   ax, bx, cx, dx, di, es
; note:
;   color is a packed 2 byte word, i.e. 0x0F0F = white!
;
h_line:
	push	bp
	mov	bp, sp

	or	bx, -1			; Mask bits
	mov	di, [bp+10]		; di = X1
	mov	cx, di
	and	cl, 0x07		; cl = X1 % 8
	shr	bh, cl			; bh = Left mask
	shr	di, 0x03		; di = X1 / 8

	mov	cx, [bp+8]		; cx = X2
	push	cx
	not	cx
	and	cl, 0x07		; cl = 7 - X2 % 8
	shl	bl, cl			; bl = Right mask
	pop	cx
	shr	cx, 0x03		; cx = X2 / 8
	sub	cx, di			; cx = Width
	jnz	.1
	and	bh, bl			; Line fits 1 in byte; AND masks
.1:	mov	ax, [bp+6]		; ax = Y
	shl	ax, 0x04		; ax = 16 * Y
	add	di, ax			; di = 16 * Y + X1 / 8
	shl	ax, 0x02		; ax = 64 * Y
	add	di, ax			; di = 80 * Y + X1  / 8
	mov	ax, 0xa000
	mov	es, ax			; es = Video segment

	xor	al, al			; Set/Reset Register
	mov	ah, [bp+4]		; ah = Color
	mov	dx, 0x03ce		; Graphics Controller Index
	out	dx, ax
	mov	ax, 0x0f01		; Enable Set/Reset Register
	out	dx, ax
	mov	ah, bh			; ah = Bit mask
	mov	al, 0x08		; Bit Mask Register
	out	dx, ax
	inc	byte [es:di]		; Write left mask

	inc	dx			; Graphics Controller Data
	jcxz	.3			; Width > 0?
	inc	di
	dec	cx
	jz	.2			; --Pixels left > 0?
	mov	al, 0xff		; Bit mask: fill 8 pixels
	out	dx, al
rep	stosb
.2:	mov	al, bl			; al = Bit mask
	out	dx, al
	inc	byte [es:di]		; Write right mask
.3:	mov	al, 0xff		; Default bit mask
	out	dx, al
	mov	ax, 0x0001		; Enable Set/Reset Register
	dec	dx			; Graphics Controller Index
	out	dx, ax

	pop	bp
	ret

;
; v_line(x, y1, y2 : word; color : byte)
; destroys:
;   ax, bx, cx, dx, di, es
;
v_line:
	push	bp
	mov	bp, sp

	mov	bh, 0x80		; Mask bit
	mov	di, [bp+10]		; di = X
	mov	cx, di			; cx = X
	and	cl, 0x07		; cl = X % 8
	shr	bh, cl			; bh = bit mask
	shr	di, 0x03		; di = X / 8

	mov	ax, [bp+8]		; ax = Y1
	mov	cx, [bp+6]		; cx = Y2
	sub	cx, ax			; cx = Height
	shl	ax, 0x04		; ax = 16 * Y1
	add	di, ax			; di = 16 * Y1 + X / 8
	shl	ax, 0x02		; ax = 64 * Y1
	add	di, ax			; di = 80 * Y1 + X / 8
	mov	ax, 0xa000
	mov	es, ax			; es = Video segment

	mov	dx, 0x03ce		; Graphics Controller Index
	mov	ah, [bp+4]		; ah = Color
	xor	al, al			; Set/Reset Register
	out	dx, ax
	mov	ax, 0x0f01		; Enable Set/Reset Register
	out	dx, ax
	mov	ah, bh			; ah = Bit mask
	mov	al, 0x08		; Bit Mask Register
	out	dx, ax

.1:	inc	byte [es:di]		; Write pixel
	add	di, 0x0050		; Next scanline
	dec	cx			; Decrease count
	jnz	.1			; Zero?

	mov	al, 0xff		; Default bit mask
	inc	dx			; Graphics Controller Data
	out	dx, al
	mov	ax, 0x0001		; Enable Set/Reset Register
	dec	dx			; Graphics Controller Index
	out	dx, ax

	pop	bp
	ret